#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "main.h"
#include "usb_cdc.h"


void init(void) 
{
	CLK.PSCTRL = 0x00;  // no division on peripheral clock

	// Power reduction: Stop unused peripherals
	PR.PRGEN = PR_AES_bm|PR_EBI_bm;												// Stop: AES, EBI
	PR.PRPA  =           PR_AC_bm|PR_DAC_bm;									// Stop: DAC, ADC, AC
	PR.PRPB  =           PR_AC_bm|PR_DAC_bm;									// Stop: DAC, ADC, AC
	PR.PRPC  = PR_TWI_bm|PR_USART1_bm                                   ;		// Stop: TWI, USART0, USART1, SPI, HIRES
	PR.PRPD  = PR_TWI_bm|PR_USART1_bm|             PR_SPI_bm|PR_HIRES_bm;		// Stop: TWI, USART0, USART1, SPI, HIRES
	PR.PRPE  = PR_TWI_bm|PR_USART1_bm|PR_USART0_bm|PR_SPI_bm            ;		// Stop: TWI, USART0, USART1, SPI, HIRES

	PMIC.CTRL = PMIC_LOLVLEN_bm | PMIC_MEDLVLEN_bm | PMIC_HILVLEN_bm;

	Config32MHzClock();
	usb_init();
	sei();
}

int main(void)
{
	unsigned char res;
	
	init();

	// Signale fr Datenanforderung
	cdc_rxb.flag=1;
	cdc_rxb.len=0;
	cdc_rxb.bytes=0;

	cdc_txb.flag=0;
	cdc_txb.len=0;
	cdc_txb.bytes=0;

	while(1)
	{
		// manage usb-task for the endpoints (in loop)
		EP_DEF_in(ep_out);
		EP_DEF_out(ep_in);
		EP_DEF_out(ep_note);

		if(!cdc_rxb.flag) // Daten verarbeiten, falls Flag gelscht
		{
			if(cdc_rxb.bytes==cdc_rxb.len)
			{
				// all data prcessed; send the data in txd and wait for new data
				// only if out-data are processed
				if(!cdc_txb.flag)
				{
					cdc_txb.len=cdc_txb.bytes;
					cdc_txb.flag=1;
					cdc_rxb.flag=1;
					// blink led for receiving data
					// led_blink_flag|=LED_BLINK_RED;
				}
			}
			else
			{
				// process data
				res=cdc_rxb.data[cdc_rxb.bytes];
				cdc_txb.data[cdc_txb.bytes]=res+1;
				// process here the data (res)
				// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
				
				cdc_rxb.bytes+=1;
				cdc_txb.bytes+=1;
			}
		}
	}
	return 0;
}


void Config32MHzClock(void)
{
	unsigned char tmp;

	// get USBRCOSC
	NVM.CMD  = NVM_CMD_READ_CALIB_ROW_gc;
	tmp = pgm_read_byte(offsetof(NVM_PROD_SIGNATURES_t, USBRCOSC));
	NVM_CMD = NVM_CMD_NO_OPERATION_gc;  // Clean up NVM Command register.

	// enable DFLL for 32MHz osz and trim to 48MHz sync with USB start of frame
	OSC.DFLLCTRL = OSC_RC32MCREF_USBSOF_gc;
	DFLLRC32M.CALB = tmp;
	DFLLRC32M.COMP1 = 0x1B; //Xmega AU manual, 4.17.19
	DFLLRC32M.COMP2 = 0xB7;
	DFLLRC32M.CTRL = DFLL_ENABLE_bm;

	// enable 32 MHZ osz (trimmed to 48MHZ for usb)
	CCP = CCP_IOREG_gc; //Security Signature to modify clock
	OSC.CTRL = OSC_RC32MEN_bm| OSC_RC2MEN_bm; // enable internal 32MHz oscillator
	
	while(!(OSC.STATUS & OSC_RC32MRDY_bm)); // wait for oscillator ready
	
	OSC.PLLCTRL = OSC_PLLSRC_RC2M_gc | 16; // 2MHz * 16 = 32MHz
	
	CCP = CCP_IOREG_gc;
	OSC.CTRL = OSC_RC32MEN_bm | OSC_PLLEN_bm | OSC_RC2MEN_bm ; // Enable PLL
	
	while(!(OSC.STATUS & OSC_PLLRDY_bm)); // wait for PLL ready
	
	DFLLRC2M.CTRL = DFLL_ENABLE_bm;

	// use PLL as systemclk
	CCP = CCP_IOREG_gc; /* allow changing CLK.CTRL */
	CLK.CTRL = CLK_SCLKSEL_PLL_gc; // use PLL output as system clock
	//
}
